'use strict';

const helper    = require('./test-helper');
const expect    = require('chai').expect;
const miniprism = require('../src/miniprism');
const e         = helper.e;

describe('inlines', function() {
  it('parses ems', function() {
    const source = 'foo *bar* baz';
    const actual = miniprism.highlight(source, 'markdown');

    expect(actual).to.eql(e(`
      .md-p.md-line>
        {foo }+
        (span.md-em.md-has-folding>
          span.md-em-marker.md-folding{*}+
          {bar}+
          span.md-em-marker.md-folding{*})+
        { baz}
    `));
  });

  it('parses strongs', function() {
    const source = 'foo **bar** baz';
    const actual = miniprism.highlight(source, 'markdown');

    expect(actual).to.eql(e(`
      .md-p.md-line>
        {foo }+
        (span.md-strong.md-has-folding>
          span.md-strong-marker.md-folding{**}+
          {bar}+
          span.md-strong-marker.md-folding{**})+
        { baz}
    `));
  });

  it('parses links', function() {
    var source = 'Go to: [Example page](https://www.example.com).';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql(e(`
      .md-p.md-line>
        {Go to: }+
        (span.md-link.md-has-folding>
          span.md-bracket-start.md-folding{[}+
          span.md-ref{Example page}+
          span.md-bracket-end.md-folding{]}+
          span.md-paren-start.md-folding{(}+
          span.md-href.md-folding{https:&#x2F;&#x2F;www.example.com}+
          span.md-paren-end.md-folding{)})+
        {.}
    `));
  });

  [['em', '*'], ['strong', '**']].forEach(([type, marker]) => {
    it(`parses links in ${type}s`, function() {
      const source = `Go to: ${marker}[Example page](https://www.example.com)${marker}.`;
      const actual = miniprism.highlight(source, 'markdown');

      expect(actual).to.eql(e(`
        .md-p.md-line>
          {Go to: }+
          (span.md-${type}.md-has-folding>
            span.md-${type}-marker.md-folding{${marker}}+
            (span.md-link.md-has-folding>
              .md-bracket-start.md-folding{[}+
              .md-ref{Example page}+
              .md-bracket-end.md-folding{]}+
              .md-paren-start.md-folding{(}+
              .md-href.md-folding{https:&#x2F;&#x2F;www.example.com}+
              .md-paren-end.md-folding{)})+
            span.md-${type}-marker.md-folding{${marker}})+
          {.}
      `));
    });
  });

  it('does not parse links with empty refs', function() {
    var source = 'Go to: [](https://www.example.com).';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to
      .eql('<div class="md-p md-line">Go to: [](https:&#x2F;&#x2F;www.example.com).</div>');
  });

  it('does not parse links with empty hrefs', function() {
    var source = 'Go to: [example]().';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to
      .eql('<div class="md-p md-line">Go to: [example]().</div>');
  });

  it('parses link references', function() {
    var source = 'This is a [link][this_is_the_reference].';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">This is a ',
        '<span class="md-link-ref md-has-folding">',
          '<span class="md-ref-text">',
            '<span class="md-bracket-start md-folding">[</span>',
            '<span class="md-ref">link</span>',
            '<span class="md-bracket-end md-folding">]</span>',
          '</span>',
          '<span class="md-ref-value md-folding">',
            '<span class="md-bracket-start">[</span>',
            '<span class="md-ref-href">this_is_the_reference</span>',
            '<span class="md-bracket-end">]</span>',
          '</span>',
        '</span>.',
      '</div>'
    ].join(''));
  });

  it('parses inline code', function() {
    var source = 'foo `bar` baz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">foo ',
        '<span class="md-inline-code md-has-folding">',
          '<span class="md-inline-code-marker md-folding">`</span>',
          'bar',
          '<span class="md-inline-code-marker md-folding">`</span>',
        '</span>',
      ' baz</div>'
    ].join(''));
  });

  it('parses consecutive inline code separately', function() {
    const source = '`foo``bar`';
    const actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql(e([
      '.md-p.md-line>',
        '(span.md-inline-code.md-has-folding>',
          'span.md-inline-code-marker.md-folding{`}+',
          '{foo}+',
          'span.md-inline-code-marker.md-folding{`})+',
        '(span.md-inline-code.md-has-folding>',
          'span.md-inline-code-marker.md-folding{`}+',
          '{bar}+',
          'span.md-inline-code-marker.md-folding{`})+',
    ].join('')));
  });

  it('does not parse ``` as inline code', function() {
    var source = 'foo ``` baz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql('<div class="md-p md-line">foo ``` baz</div>');
  });

  it('parses inline code with backticks', function() {
    var source = 'foo ``foo`bar`` baz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">foo ',
        '<span class="md-inline-code md-has-folding">',
          '<span class="md-inline-code-marker md-folding">``</span>',
          'foo`bar',
          '<span class="md-inline-code-marker md-folding">``</span>',
        '</span>',
      ' baz</div>'
    ].join(''));
  });

  it('escapes contents', function() {
    var source = '> foo <bar&"\'\/';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-blockquote md-line">',
        '<span class="md-blockquote-marker">&gt; </span>',
        'foo &lt;bar&amp;&quot;&#x27;&#x2F;',
      '</div>'
    ].join(''));
  });
});
